@@ -38,10 +38,20 @@
 
 #include <iostream.h>
 #include <string.h>
+#include <fstream>
 
 #include <IV-2_6/_enter.h>
 
+//#define LEAKCHECK
+
+#ifdef LEAKCHECK
+#include <ivstd/leakchecker.h>
+LeakChecker AttributeValueListchecker("AttributeValueList");
+#endif
+
+
 /*****************************************************************************/
+using std::cerr;
 
 int AttributeList::_symid = -1;
 
@@ -203,11 +213,11 @@ ostream& operator<< (ostream& out, const AttributeList& al) {
 	Attribute* attr = attrlist->GetAttr(i);
 	out << " :" << attr->Name() << " ";
 
-	char* string;
 	AttributeValue* attrval = attr->Value();
 #if 1
 	out << *attrval;
 #else
+	char* string;
         switch(attr->Value()->type()) {
 	    case AttributeValue::SymbolType:
 	        out << attrval->symbol_ptr();
@@ -298,6 +308,9 @@ void AttributeList::clear() {
 /*****************************************************************************/
 
 AttributeValueList::AttributeValueList (AttributeValueList* s) {
+#ifdef LEAKCHECK
+    AttributeValueListchecker.create();
+#endif
     _alist = new AList;
     _count = 0;
     if (s != nil) {
@@ -311,6 +324,9 @@ AttributeValueList::AttributeValueList (AttributeValueList* s) {
 }
 
 AttributeValueList::~AttributeValueList () { 
+#ifdef LEAKCHECK
+    AttributeValueListchecker.destroy();
+#endif
     if (_alist) {
         ALIterator i;
 	for (First(i); !Done(i); Next(i)) {
@@ -425,8 +441,14 @@ ostream& operator<< (ostream& out, const AttributeValueList& al) {
 	    case AttributeValue::DoubleType:
 	        out << attrval->double_ref();
 	        break;
+	    case AttributeValue::BooleanType:
+	        out << attrval->boolean_ref();
+	        break;
+	    case AttributeValue::ArrayType:
+	        out << *attrval->array_ref();
+	        break;
             default:
-		out << "Unknown type";
+		out << "nil";
 	        break;
 	}
 
@@ -447,3 +469,39 @@ void AttributeValueList::clear() {
   }
 }
 
+AttributeValue* AttributeValueList::Get(unsigned int index) {
+  if (Number()<=index) return nil;
+  Iterator it;
+  First(it);
+  for (int i=0; i<index; i++) Next(it);
+  return GetAttrVal(it);
+}
+
+AttributeValue* AttributeValueList::Set(unsigned int index, AttributeValue* av) {
+  if (Number()<=index) {
+    Iterator it;
+    Last(it);
+    int padding = index-Number();
+    for (int i=0; i<padding; i++) Append(new AttributeValue());
+    Append(av);
+    return nil;
+  }
+  else {
+    Iterator it;
+    First(it);
+    for (int i=0; i<index; i++) Next(it);
+    AttributeValue* oldv = Replace(it, av);
+    return oldv;
+  }
+}
+
+AttributeValue* AttributeValueList::Replace (ALIterator& i, AttributeValue* av) {
+    AList* doomed = Elem(i);
+    AttributeValue* removed = GetAttrVal(i);
+    Next(i);
+    _alist->Remove(doomed);
+    delete doomed;
+    Elem(i)->Append(new AList(av));
+    return removed;
+}	
+    
