@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2001 Scott E. Johnston
+ * Copyright (c) 2001-2007 Scott E. Johnston
  * Copyright (c) 2000 IET Inc.
  * Copyright (c) 1994-1998 Vectaport Inc.
  *
@@ -23,6 +23,16 @@
  * 
  */
 
+#include <ctype.h>
+#include <iostream.h>
+#include <string.h>
+#include <strstream>
+#include <unistd.h>
+
+#if __GNUC__>=3
+#include <fstream.h>
+#endif
+
 #include <ComTerp/comhandler.h>
 
 #include <ComTerp/_comterp.h>
@@ -56,6 +66,7 @@
 #include <OS/math.h>
 
 #include <ctype.h>
+#include <errno.h>
 #include <iostream.h>
 #include <string.h>
 #include <strstream>
@@ -64,9 +75,21 @@
 #include <fstream.h>
 #endif
 
+// #define LEAKCHECK
+
+#ifdef LEAKCHECK
+#include <ivstd/leakchecker.h>
+extern LeakChecker AttributeValuechecker;
+#endif
+
 #define TITLE "ComTerp"
 #define STREAM_MECH
 
+extern int _detail_matched_delims;
+
+using std::cerr;
+using std::cout;
+
 implementTable(ComValueTable,int,void*)
 
 ComTerp* ComTerp::_instance = nil;
@@ -139,7 +162,10 @@ void ComTerp::init() {
     _trace_mode = 0;
     _npause = 0;
     _stepflag = 0;
-
+    _echo_postfix = 0;
+    _delim_func = 0;
+    _running = 0;
+    _muted = 0;
 }
 
 
@@ -159,12 +185,15 @@ const ComValue* ComTerp::stack(unsigned int &top) const {
 }
 
 boolean ComTerp::read_expr() {
-    unsigned int toklen, tokstart;
+    check_parser_client();
     int status = parser (_inptr, _infunc, _eoffunc, _errfunc, (FILE*)_outptr, _outfunc,
 			 _buffer, _bufsiz, &_bufptr, _token, _toksiz, &_linenum,
 			 &_pfbuf, &_pfsiz, &_pfnum);
 
     _pfoff = 0;
+    save_parser_client();    
+    postfix_echo();
+
     return status==0 && _pfbuf[_pfnum-1].type != TOK_EOF && _buffer[0] != '\0';
 }
 
@@ -222,10 +251,6 @@ int ComTerp::eval_expr(ComValue* pfvals, int npfvals) {
 
 void ComTerp::eval_expr_internals(int pedepth) {
   static int step_symid = symbol_add("step");
-  static ComFunc* stepfunc = nil;
-  if (!stepfunc)
-    stepfunc = new ComterpStepFunc(this);
-
   ComValue sv = pop_stack(false);
   
   if (sv.type() == ComValue::CommandType) {
@@ -248,8 +273,10 @@ void ComTerp::eval_expr_internals(int pedepth) {
       }
     if (has_streams) {
       AttributeValueList* avl = new AttributeValueList();
-      for(int i=0; i<sv.narg()+sv.nkey(); i++)
-	avl->Prepend(new AttributeValue(pop_stack(true)));
+      for(int i=0; i<sv.narg()+sv.nkey(); i++) {
+	ComValue topval(pop_stack(true));
+	avl->Prepend(new AttributeValue(topval));
+      }
       ComValue val(sv.obj_val(), avl);
       val.stream_mode(1); // for external use
       push_stack(val);
@@ -258,6 +285,8 @@ void ComTerp::eval_expr_internals(int pedepth) {
 #endif
 
     ComFunc* func = nil;
+    int nargs = sv.narg();
+    int nkeys = sv.nkey();
     if (_func_for_next_expr) {
       func = _func_for_next_expr;
       _func_for_next_expr = nil;
@@ -265,8 +294,12 @@ void ComTerp::eval_expr_internals(int pedepth) {
       func->push_funcstate(1, 0, pedepth, func->funcid());
     } else {   
       func = (ComFunc*)sv.obj_val();
-      func->push_funcstate(sv.narg(), sv.nkey(), 
-			   pedepth, sv.command_symid());
+      if (_delim_func && sv.nids()!=1) {
+	ComValue nameval(sv.command_symid(), ComValue::SymbolType);
+	push_stack(nameval);  // this assumes it will be immediately popped off the stack
+	nargs++;
+      }
+      func->push_funcstate(nargs, nkeys, pedepth, func->funcid());
     }
 
     /* output execution trace */
@@ -274,7 +307,7 @@ void ComTerp::eval_expr_internals(int pedepth) {
       for(int i=0; i<pedepth; i++) cerr << "    ";
       cerr << symbol_pntr(sv.command_symid());
       if (func->post_eval()) 
-	cerr << ": nargs=" << sv.narg() << " nkeys=" << sv.nkey() << "\n";
+	cerr << ": nargs=" << nargs << " nkeys=" << nkeys << "\n";
       else {
 	int ntotal = func->nargs() + func->nkeys();
 	for(int i=0; i<ntotal; i++) {
@@ -302,12 +335,19 @@ void ComTerp::eval_expr_internals(int pedepth) {
       static int pause_symid = symbol_add("pause");
       ComValue pausekey(pause_symid, 0, ComValue::KeywordType);
       push_stack(pausekey);
-      stepfunc->push_funcstate(0,1, pedepth, step_symid);
-      stepfunc->execute();
-      stepfunc->pop_funcstate();
+      ComterpStepFunc stepfunc(this);
+      stepfunc.push_funcstate(0,1, pedepth, step_symid);
+      stepfunc.execute();
+      stepfunc.pop_funcstate();
       pop_stack();
     }
 
+    int stack_base = _stack_top;
+    if (!func->post_eval()) 
+      stack_base -= nargs+nkeys;
+    else
+      stack_base -= 1;
+
     func->execute();
     func->pop_funcstate();
 
@@ -315,6 +355,11 @@ void ComTerp::eval_expr_internals(int pedepth) {
       push_stack(ComValue::blankval());
       _just_reset = false;
     }
+
+    if (stack_base+1 < _stack_top)
+      fprintf(stderr, "func \"%s\" failed to push a single value on stack\n", symbol_pntr(func->funcid()));
+    else if (stack_base+1 > _stack_top)
+      fprintf(stderr, "func \"%s\" pushed more than a single value on stack\n", symbol_pntr(func->funcid()));
     
   } else if (sv.type() == ComValue::SymbolType) {
 
@@ -347,11 +392,12 @@ void ComTerp::eval_expr_internals(int pedepth) {
     
   } else if (sv.is_object(Attribute::class_symid())) {
     
+
     push_stack(*((Attribute*)sv.obj_val())->Value());
     
   } else if (sv.type() == ComValue::BlankType) {
 
-    eval_expr_internals(pedepth);
+    if (!stack_empty()) eval_expr_internals(pedepth);
 
   } else {  /* everything else*/
     
@@ -410,8 +456,8 @@ void ComTerp::load_sub_expr() {
 	stack_top(0).array_val()->nested_insert(true);
       } else if (stack_top(0).is_symbol()) {
         AttributeValue* av = lookup_symval(&stack_top(0));
-	if (av->is_array()) av->array_val()->nested_insert(true);
-      }
+	if (av && av->is_array()) av->array_val()->nested_insert(true);
+      } 
     }
     _pfoff++;
     if (stack_top().type() == ComValue::CommandType && 
@@ -668,8 +714,29 @@ void ComTerp::token_to_comvalue(postfix_token* token, ComValue* sv) {
     unsigned int command_symid = sv->int_val();
     localtable()->find(vptr, command_symid);
 
+    /* handle case where symbol has matched parens, and things are set up to invoke a delim-specific func. */
+    if (!vptr && _delim_func && sv->nids() != 1) {
+      if (sv->nids() == TOK_RPAREN) {
+	static int parens_symid =  symbol_add("()");
+	localtable()->find(vptr, parens_symid);
+      }
+      if (sv->nids() == TOK_RBRACKET) {
+	static int brackets_symid =  symbol_add("[]");
+	localtable()->find(vptr, brackets_symid);
+      }
+      else if (sv->nids() == TOK_RBRACE) {
+	static int braces_symid =  symbol_add("{}");
+	localtable()->find(vptr, braces_symid);
+      }
+      else if (sv->nids() == TOK_RANGBRACK) {
+	static int anglebrackets_symid =  symbol_add("<>");
+	localtable()->find(vptr, anglebrackets_symid);
+      }
+      command_symid = sv->symbol_val();
+    }
+
     /* handle case where symbol has arguments/keywords, but is not defined */
-    if (!vptr && (sv->narg() || sv->nkey())) {
+    else if (!vptr && (sv->narg() || sv->nkey())) {
       static int nil_symid = symbol_add("nil");
       localtable()->find(vptr, nil_symid);
     }
@@ -694,6 +761,12 @@ void ComTerp::push_stack(ComValue& value) {
 	}
     } 
     _stack_top++;
+
+    if (_stack_top<0) {
+      fprintf(stderr, "warning: comterp stack still empty after push\n");
+      return;
+    }
+
     ComValue* sv = _stack + _stack_top;
     *sv = ComValue(value);
     if (sv->type() == ComValue::KeywordType)
@@ -732,18 +805,29 @@ void ComTerp::decr_stack(int n) {
     for (int i=0; i<n && _stack_top>=0; i++) {
         ComValue& stacktop = _stack[_stack_top--];
 	stacktop.AttributeValue::~AttributeValue();
+        #ifdef LEAKCHECK // destructor called where constructor never called
+	AttributeValuechecker.create();
+        #endif
     }
 }
 
-ComValue& ComTerp::pop_stack(boolean lookupsym) {
+ComValue ComTerp::pop_stack(boolean lookupsym) {
   if (!stack_empty()) {
     ComValue& stacktop = _stack[_stack_top--];
+    ComValue topval(stacktop);
+    stacktop.AttributeValue::~AttributeValue();
+    #ifdef LEAKCHECK  // destructor called where constructor never called
+    AttributeValuechecker.create();
+    #endif
     if (lookupsym)
-      return lookup_symval(stacktop);
+      return lookup_symval(topval);
     else 
-      return stacktop;
-  } else
+      return topval;
+
+  } else {
+    cerr << "stack empty, blank returned\n";
     return ComValue::blankval();
+  }
 }
 
 ComValue& ComTerp::lookup_symval(ComValue& comval) {
@@ -872,11 +956,13 @@ void ComTerp::quitflag(boolean flag) {
 }
 
 int ComTerp::run(boolean one_expr, boolean nested) {
+  int old_runflag = running();
+  running(true);
+
   int status = 1;
   _errbuf[0] = '\0';
   char errbuf_save[BUFSIZ];
   errbuf_save[0] = '\0';
-  
 
 #if __GNUC__<3
   filebuf fbuf;
@@ -885,7 +971,7 @@ int ComTerp::run(boolean one_expr, boolean nested) {
     fbuf.attach(fd);
   } else
     fbuf.attach(fileno(stdout));
-#elif __GNUC__==3 && __GNUC_MINOR__<1
+#elif (__GNUC__==3 && __GNUC_MINOR__<1) || __GNUC__>3 || defined(__CYGWIN__)
   fileptr_filebuf fbuf(handler() && handler()->wrfptr() 
 	       ? handler()->wrfptr() : stdout, 
 	       ios_base::out);
@@ -896,6 +982,7 @@ int ComTerp::run(boolean one_expr, boolean nested) {
 #endif
   ostream out(&fbuf);
   boolean eolflag = false;
+  boolean errorflag = false;
 
   while (!eof() && !quitflag() && !eolflag) {
     
@@ -906,11 +993,12 @@ int ComTerp::run(boolean one_expr, boolean nested) {
       if (top_before == _stack_top)
 	status = 2;
       err_str( _errbuf, BUFSIZ, "comterp" );
+      errno = 0;
       if (strlen(_errbuf)==0) {
 	if (quitflag()) {
 	  status = -1;
 	  break;
-	} else if (!func_for_next_expr() && val_for_next_func().is_null()) {
+	} else if (!func_for_next_expr() && val_for_next_func().is_null() && !muted()) {
 	  print_stack_top(out);
 	  out << "\n"; out.flush(); 
 	}
@@ -922,6 +1010,7 @@ int ComTerp::run(boolean one_expr, boolean nested) {
     } else {
       err_str( _errbuf, BUFSIZ, "comterp" );
       if (strlen(_errbuf)>0) {
+	errorflag = true;
 	out << _errbuf << "\n"; out.flush();
 	strcpy(errbuf_save, _errbuf);
 	_errbuf[0] = '\0';
@@ -930,12 +1019,20 @@ int ComTerp::run(boolean one_expr, boolean nested) {
         if (errbuf_save[0]) strcpy(_errbuf, errbuf_save);
       }
     }
-    if (!nested) 
-      _stack_top = -1;
+    if (!nested)
+      decr_stack(_stack_top+1);
     if (one_expr) break;
   }
   if (status==1 && _pfnum==0) status=2;
-  if (nested && status!=2) _stack_top--;
+  if (status==1 && !errorflag) status=3;
+  #if 0 // has to be dealt with a different way
+  if (nested && status!=2) pop_stack();
+  #endif
+  if (errno == EPIPE) {
+    status = -1;
+    fprintf(stderr, "broken pipe detected: comterp quit\n");
+  }
+  running(old_runflag);
   return status;
 }
 
@@ -1042,6 +1139,7 @@ void ComTerp::add_defaults() {
     add_command("run", new RunFunc(this));
 
     add_command("help", new HelpFunc(this));
+    add_command("optable", new OptableFunc(this));
     add_command("trace", new ComterpTraceFunc(this));
     add_command("pause", new ComterpPauseFunc(this));
     add_command("step", new ComterpStepFunc(this));
@@ -1068,6 +1166,7 @@ void ComTerp::add_defaults() {
 
     add_command("print", new PrintFunc(this));
 
+    add_command("usleep", new USleepFunc(this));
 #ifdef HAVE_ACE
     add_command("timeexpr", new TimeExprFunc(this));
 #endif
@@ -1076,6 +1175,7 @@ void ComTerp::add_defaults() {
     add_command("shell", new ShellFunc(this));
     add_command("quit", new QuitFunc(this));
     add_command("exit", new ExitFunc(this));
+    add_command("mute", new MuteFunc(this));
   }
 }
 
@@ -1089,6 +1189,9 @@ AttributeList* ComTerp::get_attributes() { return _alist;}
 
 
 int ComTerp::runfile(const char* filename) {
+    int old_runflag = running();
+    running(true);
+
     /* save tokens to restore after the file has run */
     int toklen;
     postfix_token* tokbuf = copy_postfix_tokens(toklen);
@@ -1150,6 +1253,7 @@ int ComTerp::runfile(const char* filename) {
     } else
         push_stack(ComValue::nullval());
 
+    running(old_runflag);
     return status;
 }
 
@@ -1413,5 +1517,58 @@ void ComTerp::push_servstate() {
   *ctss = cts_state;
 }
 
+boolean ComTerp::stack_empty() { return _stack_top<0; }
 
+void ComTerp::postfix_echo() {
+  if (!_echo_postfix) return;
+  // print everything in the _pfbuf for this function
+#if __GNUC__<3
+  filebuf fbuf;
+  if (handler()) {
+    int fd = Math::max(1, handler()->get_handle());
+    fbuf.attach(fd);
+  } else
+    fbuf.attach(fileno(stdout));
+#else
+  fileptr_filebuf fbuf(handler() && handler()->wrfptr()
+	       ? handler()->wrfptr() : stdout, ios_base::out);
+#endif
+  ostream out(&fbuf);
+ 
+  boolean oldbrief = brief();
+  brief(true);
+
+  ComValue val;
+  for (int i=0; i<_pfnum; i++) {
+    ComValue val;
+    token_to_comvalue(_pfbuf+i, &val);
+    val.comterp(this);
+    out << val;
+    if (val.is_type(AttributeValue::CommandType) ||
+       (_detail_matched_delims && val.is_type(AttributeValue::SymbolType) && 
+	val.nids() >= TOK_RPAREN )) {
+      if (!_detail_matched_delims) {
+	out << "[" << val.narg() << "|" << val.nkey() << "]";
+	ComFunc* func = (ComFunc*)val.obj_val();
+	if (func->post_eval()) out << "*";
+      } else {
+	char ldelim, rdelim;
+	if (val.nids()==TOK_RPAREN) {ldelim = '('; rdelim = ')'; }
+	else if (val.nids()==TOK_RBRACKET) {ldelim = '['; rdelim = ']'; }
+	else if (val.nids()==TOK_RBRACE) {ldelim = '{'; rdelim = '}'; }
+	else if (val.nids()==TOK_RANGBRACK) {ldelim = '<'; rdelim = '>'; }
+	else {ldelim = ':'; rdelim = 0x0;};
+	out << ldelim << val.narg();
+	if (rdelim) out << rdelim;
+      }
+    }
+    else if (val.is_type(AttributeValue::SymbolType) && 
+	     (val.narg() || val.nkey()))
+      out << "{" << val.narg() << "|" << val.nkey() << "}";
+    else if (val.is_type(AttributeValue::KeywordType))
+      out << "(" << val.keynarg_val() << ")";
+    out << ((i==_pfnum-1) ? "\n" : " ");
+  }
+  brief(oldbrief);
+}
 
