@@ -23,7 +23,6 @@
  */
 
 #include <ComTerp/comhandler.h>
-
 #include <ComTerp/_comterp.h>
 #include <ComTerp/_comutil.h>
 #include <ComTerp/comterpserv.h>
@@ -64,8 +63,12 @@ ComTerpServ::ComTerpServ(int bufsize, int fd)
 
     /* inform the parser which infunc is the oneshot infunc */
     _oneshot_infunc = (infuncptr)&s_fgets;
+  
+    /* initialize shadow copy too */
+    __oneshot_infunc = (infuncptr)&s_fgets;
 
     _logger_mode = 0;
+    _delete_later = 0;
 }
 
 ComTerpServ::~ComTerpServ() {
@@ -152,9 +155,15 @@ char* ComTerpServ::fd_fgets(char* s, int n, void* serv) {
     fbuf.attach(fd);
     istream in (&fbuf);
     in.gets(&instr);
-#elif __GNUC__==3 && __GNUC_MINOR__<1
+#elif (__GNUC__==3 && __GNUC_MINOR__<1)
     char instr[BUFSIZ];
-    FILE* ifptr = fdopen(fd, "r");
+    FILE* ifptr = fd==0 ? stdin : fdopen(fd, "r");
+    fileptr_filebuf fbuf(ifptr, ios_base::in);
+    istream in (&fbuf);
+    in.get(instr, BUFSIZ, '\n');  // needs to be generalized with <vector.h>
+#elif __GNUC__>3 || defined(__CYGWIN__)
+    char instr[BUFSIZ];
+    FILE* ifptr = fd==0 ? stdin : server->handler()->rdfptr();
     fileptr_filebuf fbuf(ifptr, ios_base::in);
     istream in (&fbuf);
     in.get(instr, BUFSIZ, '\n');  // needs to be generalized with <vector.h>
@@ -185,8 +194,8 @@ char* ComTerpServ::fd_fgets(char* s, int n, void* serv) {
     /* append a null byte */
     outstr[outpos] = '\0';
 
-#if __GNUC__==3 && __GNUC_MINOR__<1
-    if (ifptr) fclose(ifptr);
+#if (__GNUC__==3 && __GNUC_MINOR__<1)
+    if (ifptr && fd!=0) fclose(ifptr);
 #endif
 
     return s;
@@ -202,25 +211,27 @@ int ComTerpServ::fd_fputs(const char* s, void* serv) {
 #if __GNUC__<3
     filebuf fbuf;
     fbuf.attach(fd);
-#elif __GNUC__==3 && __GNUC_MINOR__<1
-    FILE* ofptr = fdopen(fd, "w");
+#elif (__GNUC__==3 && __GNUC_MINOR__<1)
+    FILE* ofptr = fd==0 ? stdout : fdopen(fd, "w");
+    fileptr_filebuf fbuf(ofptr, ios_base::out);
+#elif __GNUC__>3 || defined(__CYGWIN__)
+    FILE* ofptr = fd==0 ? stdout : server->handler()->wrfptr();
     fileptr_filebuf fbuf(ofptr, ios_base::out);
 #else
-    fileptr_filebuf fbuf(fd, ios_base::out, false, static_cast<size_t>(BUFSIZ));
+    fileptr_filebuf fbuf(fd==0?1:fd, ios_base::out, false, static_cast<size_t>(BUFSIZ));
 #endif
     ostream out(&fbuf);
     for (; outpos < bufsize-1 && s[outpos]; outpos++)
 	out.put(s[outpos]);
     out.flush();
     outpos = 0;
-#if __GNUC__==3 && __GNUC_MINOR__<1
-    if (ofptr) fclose(ofptr);
+#if (__GNUC__==3 && __GNUC_MINOR__<1)
+    if (ofptr && fd!=0) fclose(ofptr);
 #endif
     return 1;
 }
 
 int ComTerpServ::run(boolean one_expr, boolean nested) {
-
     char buffer[BUFSIZ];
     char errbuf[BUFSIZ];
     errbuf[0] = '\0';
@@ -321,7 +332,10 @@ int ComTerpServ::runfile(const char* filename) {
 #if __GNUC__<3
 	        filebuf obuf(handler() ? handler()->get_handle() : 1);
 #elif __GNUC__==3 && __GNUC_MINOR__<1
-                FILE* ofptr = fdopen(handler() ? handler()->get_handle() : 1, "w"); 
+                FILE* ofptr = handler() ? fdopen(handler()->get_handle(), "w") : stdout; 
+	        fileptr_filebuf obuf(ofptr, ios_base::out);
+#elif __GNUC__>3 || defined(__CYGWIN__)
+                FILE* ofptr = handler() ? handler()->wrfptr() : stdout; 
 	        fileptr_filebuf obuf(ofptr, ios_base::out);
 #else
 		fileptr_filebuf obuf(handler()&&handler()->get_handle()>0 
@@ -331,7 +345,7 @@ int ComTerpServ::runfile(const char* filename) {
 		ostream ostr(&obuf);
 		ostr.flush();
 #if __GNUC__==3 && __GNUC_MINOR__<1
-                if (ofptr) fclose(ofptr);
+                if (ofptr && handler()) fclose(ofptr);
 #endif
 		status = -1;
 	    } else if (quitflag()) {
@@ -346,7 +360,10 @@ int ComTerpServ::runfile(const char* filename) {
 #if __GNUC__<3
 	  filebuf obuf(handler() ? handler()->get_handle() : 1);
 #elif __GNUC__==3 && __GNUC_MINOR__<1
-          FILE* ofptr = fdopen(handler() ? handler()->get_handle() : 1, "w"); 
+          FILE* ofptr = handler() ? fdopen(handler()->get_handle(), "w") : stdout; 
+	  fileptr_filebuf obuf(ofptr, ios_base::out);
+#elif __GNUC__>3 || defined(__CYGWIN__)
+          FILE* ofptr = handler() ? handler()->wrfptr() : stdout; 
 	  fileptr_filebuf obuf(ofptr, ios_base::out);
 #else
 	  fileptr_filebuf obuf(handler()&&handler()->get_handle()>0 
@@ -357,7 +374,7 @@ int ComTerpServ::runfile(const char* filename) {
 	  ostream ostr(&obuf);
 	  ostr.flush();
 #if __GNUC__==3 && __GNUC_MINOR__<1
-          if (ofptr) fclose(ofptr);
+          if (ofptr && handler()) fclose(ofptr);
 #endif
 	  status = -1;
 	}
@@ -386,7 +403,7 @@ int ComTerpServ::runfile(const char* filename) {
     return status;
 }
 
-ComValue& ComTerpServ::run(const char* expression, boolean nested) {
+ComValue ComTerpServ::run(const char* expression, boolean nested) {
     _errbuf[0] = '\0';
 
 #if 0
@@ -461,7 +478,7 @@ ComValue& ComTerpServ::run(const char* expression, boolean nested) {
     return *_errbuf ? ComValue::nullval() : pop_stack();
 }
 
-ComValue& ComTerpServ::run(postfix_token* tokens, int ntokens) {
+ComValue ComTerpServ::run(postfix_token* tokens, int ntokens) {
     _errbuf[0] = '\0';
 
 #if 0
@@ -481,7 +498,7 @@ ComValue& ComTerpServ::run(postfix_token* tokens, int ntokens) {
     eval_expr();
     err_str(_errbuf, BUFSIZ, "comterp");
 
-    ComValue& retval = *_errbuf ? ComValue::nullval() : pop_stack();
+    ComValue retval(*_errbuf ? ComValue::nullval() : pop_stack());
 #if 0
     _pfbuf = save_pfbuf;
     _pfnum = save_pfnum;
