@@ -60,6 +60,10 @@
 
 #include <strstream>
 #include <string.h>
+#include <fstream>
+#include <iostream>
+
+using std::cout;
 
 /*****************************************************************************/
 
@@ -225,6 +229,9 @@ void ComEditor::AddCommands(ComTerp* comterp) {
     comterp->add_command("pflush", new PixelFlushFunc(comterp, this));
     comterp->add_command("pclip", new PixelClipFunc(comterp, this));
     comterp->add_command("alpha", new AlphaTransFunc(comterp, this));
+
+    comterp->add_command("trans", new TransformerFunc(comterp, this));
+
 }
 
 /* virtual */ void ComEditor::ExecuteCmd(Command* cmd) {
