@@ -1,4 +1,5 @@
 /*
+ * Copyright (c) 2005 Scott E. Johnston
  * Copyright (c) 2000 IET Inc.
  * Copyright (c) 1993-1995 Vectaport Inc.
  * Copyright (c) 1989 Triple Vision, Inc.
@@ -32,9 +33,6 @@ Summary:        Lexical scanning routine to recognize C-like tokens
 History:        Written by Scott E. Johnston, March 1989
 */
 
-extern int _continuation_prompt;
-extern int _continuation_prompt_disabled;
-
 #include <stdio.h>
 #include <string.h>
 #include <ctype.h>
@@ -43,6 +41,12 @@ extern int _continuation_prompt_disabled;
 
 #include "comutil.ci"
 
+extern int _continuation_prompt;
+extern int _continuation_prompt_disabled;
+unsigned _token_state_save = TOK_WHITESPACE;
+				/* variable to save token state between calls */
+int _ignore_numerics = 0;
+
 /* MACROS */
 
 /* States for parsing floating point numbers */
@@ -152,8 +156,6 @@ unsigned double_state = FLOAT_INTEGER;
 BOOLEAN long_num = FALSE;       /* Indicates long integer to be used */
 unsigned token_state = TOK_WHITESPACE;
 				/* Internal token state variable */
-static unsigned token_state_save = TOK_WHITESPACE;
-				/* variable to save token state between calls */
 unsigned begcmt_len =           /* Number of characters in comment beginning */
    (begcmt != NULL ? strlen(begcmt) : 0 );
 unsigned endcmt_len =           /* Number of characters in comment ending */
@@ -176,10 +178,13 @@ char* infunc_retval;
 
 /* Initialize */
    *toktype = TOK_NONE;
-   *toklen = 0;
    *tokstart = 0;
-   token_state = token_state_save;
-   token_state_save = TOK_WHITESPACE;
+   if (_token_state_save != TOK_STRING)
+     *toklen = 0;
+   else
+     *toklen = strlen(token);
+   token_state = _token_state_save;
+   _token_state_save = TOK_WHITESPACE;
 
 /* Initialize if linenumber is 0 */
    if( *linenum == 0 ) {
@@ -305,8 +310,10 @@ char* infunc_retval;
 	 *bufptr = 0;
          CURR_CHAR = buffer[0];
 	 if (CURR_CHAR == '\0') {
-	   if (token_state == TOK_COMMENT)
-	     token_state_save = token_state;
+	   if (token_state == TOK_COMMENT || token_state == TOK_STRING) {
+	     _token_state_save = token_state;
+	     token[*toklen] = '\0';
+  	    }
 	    *linenum--;
 	    return  FUNCOK;
 	 }
@@ -358,7 +365,7 @@ char* infunc_retval;
             }
 
       /* Start of identifier */
-	 else if( isident( CURR_CHAR )) {
+	 else if( isident( CURR_CHAR ) || (_ignore_numerics && isdigit( CURR_CHAR))) {
 	    token_state = TOK_IDENTIFIER;
 	    TOKEN_ADD( CURR_CHAR );
 	    }
