@@ -45,6 +45,11 @@
 #include <math.h>
 #include <version.h>
 
+#include <iostream>
+#include <fstream>
+
+using std::cerr;
+
 
 /*****************************************************************************/
 
